<?php

namespace Lean\Console\Form\Storage;

use Picea\Ui\Method\{ FormInterface, FormContext, FormContextInterface };

use \Lean\Console\Lib;

use Picea\Compiler\Context;
use Psr\Http\Message\ServerRequestInterface;
use Ulmus\Common\EntityResolver;
use \Ulmus\Entity\InformationSchema\Table;

class DatabaseMigration implements FormInterface
{
    protected ? Lib\DatabaseMigrations $migration;

    public function __construct(Lib\DatabaseMigrations $migration)
    {
        $this->migration = $migration;
    }

    public function initialize(FormContextInterface $context) : void
    {
        $context->tableExist = [];

        foreach($this->migration->entities as $entity => $table) {
            $connection = $entity::resolveEntity()->sqlAdapter();
            $adapter = $connection->adapter();

            $tableName = $table->tableName();
            $databaseName = $table->databaseName();

            if ( ! $tableName || ! $databaseName ) {
                continue;
            }

            $tableEntity = $adapter->schemaTable($connection, $databaseName, $tableName);

            if ( $tableEntity ) {
                foreach($tableEntity->columns as $field => $definition) {
                 #   if ( $table->columns->filtersCollection(fn($e) => $e->name === $field)->count() === 0 ) {
                    #    $alter[$field] = $definition;
                    #}
                }
                
                $alter =false;

                if ( $alter ) {
                    $context->status[$entity] = [
                        'msg' => "alter",
                        "query" => $entity::repository()->alterSqlQuery($alter)->getSqlQuery(true),
                    ];
                }
                else {
                    # if columns are different
                    $context->status[$entity] = [
                        'msg' => "up-to-date",
                        'query' => "",
                    ];
                }
            }
            else {
                $context->status[$entity] = [
                    'msg' => "unexisting",
                    'query' => $entity::repository()->createSqlQuery()->getSqlQuery(true),
                ];

            }
        }

        if ( $context->formSent() ) {

        }
    }

    public function validate(FormContextInterface $context) : bool
    {
        return $context->valid();
    }

    public function execute(FormContextInterface $context) : void
    {
        if ( $context->create ?? false ) {
            $context->create::repository()->createTable();
        }
        elseif ( $context->createAll ?? false ) {
            foreach(explode(',', $context->createAll) as $entity) {
                $entity::repository()->createTable();
            }
        }
        elseif ( $context->alter ?? false ) {

        }

        $this->initialize($context);
    }

    public static function getContext(ServerRequestInterface $request, ? string $formName = null) : FormContextInterface
    {
        return new class($request, $formName) extends FormContext {

            public array $status = [];

            public array $actions = [];
        };
    }
}
